@@ -45,6 +45,20 @@
  *     function pointers are set to that it is a "normal" file with a
  *     file descriptor of -1.  Note: The cookie pointer is reset to NULL
  *     if the FILE struct is free'd by fclose.
+ *
+ *  Nov 21, 2002
+ *  Added internal function _wstdio_fwrite.
+ *  Jan 3, 2003
+ *  Fixed a bug in _wstdio_fwrite.
+ *
+ *  Jan 22, 2003
+ *  Fixed a bug related file position in append mode.  _stdio_fwrite now
+ *     seeks to the end of the stream when append mode is set and we are
+ *     transitioning to write mode, so that subsequent ftell() return
+ *     values are correct.
+ *  Also fix _stdio_fopen to support fdopen() with append specified when
+ *     the underlying file didn't have O_APPEND set.  It now sets the
+ *     O_APPEND flag as recommended by SUSv3 and is done by glibc.
  */
 
 /* Before we include anything, convert L_ctermid to L_ctermid_function
@@ -316,11 +330,11 @@ int putw(int w, FILE *stream)
 UNLOCKED(int,fileno,(register FILE *stream),(stream))
 {
 #ifdef __STDIO_GLIBC_CUSTOM_STREAMS
-	return ( ((stream->cookie == &(stream->filedes)) && (stream->filedes >= 0))
+	return ( (stream && (stream->cookie == &(stream->filedes)) && (stream->filedes >= 0))
 			 ? stream->filedes
 			 : (__set_errno(EBADF), -1) );
 #else  /* __STDIO_GLIBC_CUSTOM_STREAMS */
-	return (stream->filedes >= 0) ? stream->filedes : (__set_errno(EBADF), -1);
+	return ((stream && stream->filedes >= 0)) ? stream->filedes : (__set_errno(EBADF), -1);
 #endif /* __STDIO_GLIBC_CUSTOM_STREAMS */
 }
 
@@ -1502,8 +1516,8 @@ size_t _stdio_fread(unsigned char *buffer, size_t bytes, register FILE *stream)
  * error flag before/after the write process, but it doesn't seem worth
  * the trouble. */
 
-/* Like standard write, but always does a full write unless error plus
- *deals correctly with bufsize > SSIZE_MAX... not much on an issue on linux
+/* Like standard write, but always does a full write unless error, plus
+ * deals correctly with bufsize > SSIZE_MAX... not much on an issue on linux
  * but definitly could be on Elks.  Also on Elks, always loops for EINTR..
  * Returns number of bytes written, so a short write indicates an error */
 static size_t _stdio_WRITE(register FILE *stream,
@@ -1587,8 +1601,18 @@ size_t _stdio_fwrite(const unsigned char *buffer, size_t bytes,
 		stream->bufgetc =
 #endif /* __STDIO_GETC_MACRO */
 		stream->bufpos = stream->bufread = stream->bufstart;
-	}
+	} else
 #endif
+	if ((stream->modeflags & (__FLAG_WRITING|__FLAG_APPEND)) == __FLAG_APPEND) {
+		/* Append mode, but not currently writing.  Need to seek to end for proper
+		 * ftell() return values.  Don't worry if the stream isn't seekable. */
+		__offmax_t pos[1];
+		*pos = 0;
+		if (_stdio_lseek(stream, pos, SEEK_END) && (errno != EPIPE)) { /* Too big? */
+			stream->modeflags |= __FLAG_ERROR;
+			return 0;
+		}
+	}
 
 #ifdef __STDIO_PUTC_MACRO
 	/* We need to disable putc macro in case of error */
@@ -1711,6 +1735,7 @@ size_t _stdio_fwrite(const unsigned char *buffer, size_t bytes,
 
 	/* We always clear the reading flag in case at EOF. */
 	stream->modeflags &= ~(__FLAG_READING);
+
 	/* Unlike the buffered case, we set the writing flag now since we don't
 	 * need to do anything here for fflush(). */
 	stream->modeflags |= __FLAG_WRITING;
@@ -2317,17 +2342,26 @@ FILE *_stdio_fopen(const char * __restrict filename,
 	}
 
 	if (filedes >= 0) {			/* Handle fdopen trickery. */
-		/*
-		 * NOTE: it is insufficient to just check R/W/RW agreement.
-		 * We must also check for append mode agreement, as well as
-		 * largefile agreement if applicable.
-		 */
-		int i = (open_mode & (O_ACCMODE|O_APPEND|O_LARGEFILE)) + 1;
+		/* NOTE: it is insufficient to just check R/W/RW agreement.
+		 * We must also check largefile compatibility if applicable.
+		 * Also, if append mode is desired for fdopen but O_APPEND isn't
+		 * currently set, then set it as recommended by SUSv3.  However,
+		 * if append mode is not specified for fdopen but O_APPEND is set,
+		 * leave it set (glibc compat). */
+		int i = (open_mode & (O_ACCMODE|O_LARGEFILE)) + 1;
 
-		if ((i & (((int) filename) + 1)) != i) {
+		if (((i & (((int) filename) + 1)) != i)	/* Check basic agreement. */
+			|| (((open_mode & O_APPEND)
+				 && !(((int) filename) & O_APPEND)
+				 && fcntl(filedes, F_SETFL, O_APPEND)))	/* Need O_APPEND. */
+			) {
 			__set_errno(EINVAL);
 			filedes = -1;
 		}
+#ifdef __STDIO_LARGE_FILES
+		/* For later... to reflect largefile setting in stream flags. */
+		open_mode |= (((int) filename) & O_LARGEFILE);
+#endif /* __STDIO_LARGE_FILES */
 		stream->filedes = filedes;
 	} else {
 #ifdef __STDIO_LARGE_FILES
@@ -3328,7 +3362,7 @@ char *_uintmaxtostr(register char * __restrict bufend, uintmax_t uval,
 	unsigned int H, L, high, low, rh;
 #endif
 #ifndef __LOCALE_C_ONLY
-	int grouping;
+	int grouping, outdigit;
 	size_t gslen;		   /* This does not need to be initialized. */
 	const char *g;		   /* This does not need to be initialized. */
 #endif /* __LOCALE_C_ONLY */
@@ -3347,6 +3381,11 @@ char *_uintmaxtostr(register char * __restrict bufend, uintmax_t uval,
 
 #ifndef __LOCALE_C_ONLY
 	grouping = -1;
+	outdigit = 0x80 & alphacase;
+	alphacase ^= outdigit;
+#ifdef __UCLIBC_MJN3_ONLY_
+#warning implement outdigit... need digit lengths!  (put it in locale struct)
+#endif
 	if (alphacase == __UIM_GROUP) {
 		assert(base == 10);
 		if (*(g = CUR_LOCALE.grouping)
@@ -3388,7 +3427,18 @@ char *_uintmaxtostr(register char * __restrict bufend, uintmax_t uval,
 		digit = uval % base;
 		uval /= base;
 
-		*--bufend = ( (digit < 10) ? digit + '0' : digit + alphacase );
+#ifndef __LOCALE_C_ONLY
+		if (outdigit) {
+			outdigit = CUR_LOCALE.outdigit_length[digit];
+			do {
+				*--bufend = (&CUR_LOCALE.outdigit0_mb)[digit][--outdigit];
+			} while (outdigit);
+			outdigit = 1;
+		} else
+#endif
+		{
+			*--bufend = ( (digit < 10) ? digit + '0' : digit + alphacase );
+		}
     } while (uval);
 
 #else  /* ************************************************** */
@@ -3434,7 +3484,18 @@ char *_uintmaxtostr(register char * __restrict bufend, uintmax_t uval,
 		low = (low / base) + (H * rh) + (digit / base);
 		digit %= base;
 		
-		*--bufend = ( (digit < 10) ? digit + '0' : digit + alphacase );
+#ifndef __LOCALE_C_ONLY
+		if (outdigit) {
+			outdigit = CUR_LOCALE.outdigit_length[digit];
+			do {
+				*--bufend = (&CUR_LOCALE.outdigit0_mb)[digit][--outdigit];
+			} while (outdigit);
+			outdigit = 1;
+		} else
+#endif
+		{
+			*--bufend = ( (digit < 10) ? digit + '0' : digit + alphacase );
+		}
     } while (low | high);
 
 #endif /******************************************************/
@@ -3449,3 +3510,58 @@ char *_uintmaxtostr(register char * __restrict bufend, uintmax_t uval,
 
 #endif
 /**********************************************************************/
+#ifdef L__wstdio_fwrite
+
+#include <wchar.h>
+
+size_t _wstdio_fwrite(const wchar_t *__restrict ws, size_t n,
+					  register FILE *__restrict stream)
+{
+	size_t r, count;
+	char buf[64];
+	const wchar_t *pw;
+
+#ifdef __STDIO_BUFFERS
+	if (stream->filedes == -3) { /* Special case to support {v}swprintf. */
+		count = ((wchar_t *)(stream->bufend)) - ((wchar_t *)(stream->bufpos));
+		if (count > n) {
+			count = n;
+		}
+		if (count) {
+			wmemcpy((wchar_t *)(stream->bufpos), ws, count);
+			stream->bufpos = (char *)(((wchar_t *)(stream->bufpos)) + count);
+		}
+		return n;
+	}
+#endif
+
+	if (stream->modeflags & __FLAG_NARROW) {
+		stream->modeflags |= __FLAG_ERROR;
+		__set_errno(EBADF);
+		return 0;
+	}
+	stream->modeflags |= __FLAG_WIDE;
+
+	pw = ws;
+	count = 0;
+	while (n > count) {
+		r = wcsnrtombs(buf, &pw, n-count, sizeof(buf), &stream->state);
+		if (r != ((size_t) -1)) { /* No encoding errors */
+			if (!r) {
+				++r;		  /* 0 is returned when nul is reached. */
+				pw = ws + count + r; /* pw was set to NULL, so correct. */
+			}
+			if (_stdio_fwrite(buf, r, stream) == r) {
+				count = pw - ws;
+				continue;
+			}
+		}
+		break;
+	}
+
+	/* Note: The count is incorrect if 0 < _stdio_fwrite return < r!!! */
+	return count;
+}
+
+#endif
+/**********************************************************************/
